package com.sencha.gxt.cell.core.client.form;

import java.util.Set;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.InputElement;
import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.user.client.ui.impl.TextBoxImpl;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.widget.core.client.event.ParseErrorEvent;
import com.sencha.gxt.widget.core.client.event.ParseErrorEvent.HasParseErrorHandlers;
import com.sencha.gxt.widget.core.client.event.ParseErrorEvent.ParseErrorHandler;
import com.sencha.gxt.widget.core.client.form.PropertyEditor;

public abstract class ValueBaseFieldCell<T> extends FieldCell<T> implements HasParseErrorHandlers {

  public interface ValueBaseFieldAppearance extends FieldAppearance {
    XElement getInputElement(Element parent);
  }

  protected static TextBoxImpl impl = GWT.create(TextBoxImpl.class);

  @SuppressWarnings("unchecked")
  protected PropertyEditor<T> propertyEditor = (PropertyEditor<T>) PropertyEditor.DEFAULT;
  protected String name;
  protected boolean allowBlank;

  private final ValueBaseFieldAppearance appearance;
  private String emptyText;
  private boolean selectOnFocus;

  public ValueBaseFieldCell(ValueBaseFieldAppearance appearance) {
    super(appearance);
    this.appearance = appearance;
  }

  public ValueBaseFieldCell(ValueBaseFieldAppearance appearance, Set<String> consumedEvents) {
    super(appearance, consumedEvents);
    this.appearance = appearance;
  }

  public ValueBaseFieldCell(ValueBaseFieldAppearance appearance, String... consumedEvents) {
    super(appearance, consumedEvents);
    this.appearance = appearance;
  }

  @Override
  public HandlerRegistration addParseErrorHandler(ParseErrorHandler handler) {
    return addHandler(handler, ParseErrorEvent.getType());
  }

  public void applyEmptyText(Context context, XElement parent) {
    if (!hasFocus(context, parent) && emptyText != null && getText(parent).length() < 1) {
      setText(parent, emptyText);
      onEmpty(parent, true);
    }
  }

  public ValueBaseFieldAppearance getAppearance() {
    return appearance;
  }

  public String getEmptyText() {
    return emptyText;
  }

  @Override
  public InputElement getInputElement(Element parent) {
    return appearance.getInputElement(parent.<XElement> cast()).cast();
  }

  public String getName() {
    return name;
  }

  public PropertyEditor<T> getPropertyEditor() {
    return propertyEditor;
  }

  public String getText(XElement parent) {
    String s = getInputElement(parent).getValue();
    return emptyText != null && emptyText.equals(s) ? "" : s;
  }

  /**
   * Returns the field's allow blank state.
   * 
   * @return true if blank values are allowed
   */
  public boolean isAllowBlank() {
    return allowBlank;
  }

  public boolean isSelectOnFocus() {
    return selectOnFocus;
  }

  @Override
  public void onEmpty(XElement parent, boolean empty) {
    appearance.onEmpty(parent, empty);
  }

  public void select(XElement parent, int start, int length) {
    impl.setSelectionRange((com.google.gwt.user.client.Element) getInputElement(parent).cast(), start, length);
  }

  public void selectAll(XElement parent) {
    int length = getText(parent).length();
    if (length > 0) {
      select(parent, 0, length);
    }
  }

  /**
   * Sets whether a field is valid when its value length = 0 (default to true).
   * 
   * @param allowBlank true to allow blanks, false otherwise
   */
  public void setAllowBlank(boolean allowBlank) {
    this.allowBlank = allowBlank;
  }

  public void setEmptyText(Context context, XElement parent, String emptyText) {
    removeEmptyText(parent);
    this.emptyText = emptyText;
    applyEmptyText(context, parent);
  }

  @Override
  public void setName(XElement parent, String name) {
    this.name = name;
    getInputElement(parent).setName(name);
  }

  public void setPropertyEditor(PropertyEditor<T> propertyEditor) {
    this.propertyEditor = propertyEditor;
  }

  public void setSelectOnFocus(boolean selectOnFocus) {
    this.selectOnFocus = selectOnFocus;
  }

  public void setText(XElement parent, String text) {
    getInputElement(parent).setValue(text);
  }

  @Override
  protected void onBlur(Context context, XElement parent, T value, NativeEvent event) {
    super.onBlur(context, parent, value, event);
    applyEmptyText(context, parent);
  }

  @Override
  protected void onFocus(Context context, XElement parent, T value, NativeEvent event) {
    super.onFocus(context, parent, value, event);
    if (emptyText != null) {
      String v = getText(parent);
      if (emptyText.equals(v)) {
        setText(parent, "");
        select(parent, 0, 0);
      }
      removeEmptyText(parent);
    }
    if (selectOnFocus) {
      selectAll(parent);
    }
  }

  protected void removeEmptyText(XElement parent) {
    onEmpty(parent, false);
    if ("".equals(getText(parent))) {
      setText(parent, "");
    }
  }
}
