package com.sencha.gxt.data.shared.event;

import com.google.gwt.event.shared.EventHandler;
import com.google.gwt.event.shared.GwtEvent;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.event.shared.HasHandlers;
import com.sencha.gxt.data.shared.event.StoreRemoveEvent.StoreRemoveHandler;

/**
 * Indicates that an element that was visible has been removed from the Store.
 * Note that non-visible elements do not get events, as most display widgets are
 * not concerned with data which has been filtered out.
 * 
 * @param <M> the model type
 */
public class StoreRemoveEvent<M> extends StoreEvent<M, StoreRemoveHandler<M>> {
  public interface HasStoreRemoveHandler<M> extends HasHandlers {
    HandlerRegistration addStoreRemoveHandler(StoreRemoveHandler<M> handler);
  }
  public interface StoreRemoveHandler<M> extends EventHandler {
    void onRemove(StoreRemoveEvent<M> event);
  }

  private static GwtEvent.Type<StoreRemoveHandler<?>> TYPE;

  public static GwtEvent.Type<StoreRemoveHandler<?>> getType() {
    if (TYPE == null) {
      TYPE = new GwtEvent.Type<StoreRemoveHandler<?>>();
    }
    return TYPE;
  }

  private final int index;

  private final M item;

  public StoreRemoveEvent(int index, M item) {
    this.index = index;
    this.item = item;
  }

  @SuppressWarnings({"rawtypes", "unchecked"})
  @Override
  public GwtEvent.Type<StoreRemoveHandler<M>> getAssociatedType() {
    return (GwtEvent.Type) getType();
  }

  public int getIndex() {
    return index;
  }

  public M getItem() {
    return item;
  }

  @Override
  protected void dispatch(StoreRemoveHandler<M> handler) {
    handler.onRemove(this);
  }
}