package com.sencha.gxt.theme.base.client.field;

import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.InputElement;
import com.google.gwt.dom.client.LabelElement;
import com.google.gwt.resources.client.ClientBundle;
import com.google.gwt.resources.client.ImageResource;
import com.google.gwt.resources.client.ImageResource.ImageOptions;
import com.google.gwt.resources.client.ImageResource.RepeatStyle;
import com.google.gwt.safehtml.shared.SafeHtml;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.sencha.gxt.cell.core.client.form.CheckBoxCell.CheckBoxAppearance;
import com.sencha.gxt.core.client.XTemplates;
import com.sencha.gxt.core.client.dom.XDOM;
import com.sencha.gxt.core.client.dom.XElement;

public class CheckBoxDefaultAppearance extends ValueBaseFieldDefaultAppearance implements CheckBoxAppearance {

  public interface CheckBoxResources extends ValueBaseFieldResources, ClientBundle {
    @Source({"ValueBaseField.css", "CheckBox.css"})
    CheckBoxStyle css();

    @ImageOptions(repeatStyle = RepeatStyle.Horizontal)
    ImageResource textBackground();
  }
  public interface CheckBoxStyle extends ValueBaseFieldStyle {

    String checkBoxLabel();

  }

  public interface CheckTemplate extends XTemplates {
    @XTemplate(source = "Check.html")
    SafeHtml render(String type, CheckBoxStyle style, String id, String name, boolean readonly, Boolean value,
        String boxLabel);
  }

  public interface NotCheckedTemplate extends XTemplates {
    @XTemplate(source = "NotCheck.html")
    SafeHtml render(String type, CheckBoxStyle style, String id, String name, boolean readonly, Boolean value,
        String boxLabel);
  }

  private final CheckTemplate check;
  private final NotCheckedTemplate notChecked;

  protected final CheckBoxResources resources;
  protected String type = "checkbox";

  public CheckBoxDefaultAppearance() {
    this(GWT.<CheckBoxResources> create(CheckBoxResources.class));
  }

  public CheckBoxDefaultAppearance(CheckBoxResources resources) {
    super(resources);
    this.resources = resources;

    check = GWT.create(CheckTemplate.class);
    notChecked = GWT.create(NotCheckedTemplate.class);
  }

  @Override
  public XElement getInputElement(Element parent) {
    return parent.<XElement> cast().selectNode("input");
  }

  @Override
  public void onEmpty(Element parent, boolean empty) {

  }

  @Override
  public void onFocus(Element parent, boolean focus) {
    // Override method to prevent outline from being applied to check boxes on focus
  }

  @Override
  public void onValid(Element parent, boolean valid) {
    // no-op, cb is true or false...
  }

  @Override
  public void render(SafeHtmlBuilder sb, Boolean value, String name, boolean readonly, String boxLabel) {
    if (value) {
      sb.append(check.render(type, resources.css(), XDOM.getUniqueId(), name == null ? "" : name, readonly, value, boxLabel == null ? ""
          : boxLabel));
    } else {
      sb.append(notChecked.render(type, resources.css(), XDOM.getUniqueId(), name == null ? "" : name, readonly, value, boxLabel == null
          ? "" : boxLabel));
    }

  }

  @Override
  public void setBoxLabel(String boxLabel, XElement parent) {
    parent.selectNode("." + resources.css().checkBoxLabel()).<LabelElement> cast().setInnerHTML(boxLabel);
  }

  @Override
  public void setReadOnly(Element parent, boolean readOnly) {
    getInputElement(parent).<InputElement> cast().setReadOnly(readOnly);
  }

}
