package com.sencha.gxt.widget.core.client.container;

import com.gargoylesoftware.htmlunit.javascript.host.Event;
import com.google.gwt.core.client.GWT;
import com.google.gwt.dom.client.Style;
import com.google.gwt.event.dom.client.HasScrollHandlers;
import com.google.gwt.event.dom.client.ScrollEvent;
import com.google.gwt.event.dom.client.ScrollHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.i18n.client.LocaleInfo;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.google.gwt.uibinder.client.UiChild;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.ui.IsWidget;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.core.client.Style.Side;
import com.sencha.gxt.core.client.dom.DefaultScrollSupport;
import com.sencha.gxt.core.client.dom.HasScrollSupport;
import com.sencha.gxt.core.client.dom.ScrollSupport;
import com.sencha.gxt.core.client.dom.XDOM;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.core.client.util.Size;

public class CssFloatLayoutContainer extends InsertResizeContainer implements HasScrollHandlers, HasScrollSupport {

  public static class CssFloatData implements HasSize, LayoutData {

    private double width = -1;

    public CssFloatData() {

    }

    public CssFloatData(double width) {
      this.width = width;
    }

    @Override
    public double getSize() {
      return width;
    }

    /**
     * Sets the width of the column.
     * 
     * @param size the width, values <= 1 treated a percentages.
     */
    @Override
    public void setSize(double size) {
      this.width = size;
    }

  }

  public interface CssFloatLayoutAppearance {

    XElement getContainerTarget(XElement parent);

    void onInsert(Widget child);

    void onRemove(Widget child);

    void render(SafeHtmlBuilder sb);

  }

  protected boolean adjustForScroll = false;
  protected final CssFloatLayoutAppearance appearance;

  private Style.Float styleFloat = LocaleInfo.getCurrentLocale().isRTL() ? Style.Float.RIGHT : Style.Float.LEFT;
  private ScrollSupport scrollSupport;

  public CssFloatLayoutContainer() {
    this(GWT.<CssFloatLayoutAppearance> create(CssFloatLayoutAppearance.class));
  }

  public CssFloatLayoutContainer(CssFloatLayoutAppearance appearance) {
    this.appearance = appearance;
    SafeHtmlBuilder sb = new SafeHtmlBuilder();
    appearance.render(sb);
    setElement(XDOM.create(sb.toSafeHtml()));
  }

  @UiChild(tagname = "child")
  public void add(IsWidget child, CssFloatData layoutData) {
    if (child != null) {
      child.asWidget().setLayoutData(layoutData);
    }
    super.add(child);
  }

  @Override
  public HandlerRegistration addScrollHandler(ScrollHandler handler) {
    DOM.sinkEvents(getContainerTarget(), Event.SCROLL | DOM.getEventsSunk(getContainerTarget()));
    return addDomHandler(handler, ScrollEvent.getType());
  }

  @Override
  public ScrollSupport getScrollSupport() {
    if (scrollSupport == null) {
      scrollSupport = new DefaultScrollSupport(getContainerTarget());
    }
    return scrollSupport;
  }

  public Style.Float getStyleFloat() {
    return styleFloat;
  }

  public void insert(IsWidget w, int beforeIndex, CssFloatData layoutData) {
    if (w != null) {
      w.asWidget().setLayoutData(layoutData);
    }
    super.insert(w, beforeIndex);
  }

  /**
   * Returns true if adjust for scroll is enabled.
   * 
   * @return the adjust for scroll state
   */
  public boolean isAdjustForScroll() {
    return adjustForScroll;
  }

  /**
   * True to adjust the container width calculations to account for the scroll
   * bar (defaults to false).
   * 
   * @param adjustForScroll the adjust for scroll state
   */
  public void setAdjustForScroll(boolean adjustForScroll) {
    this.adjustForScroll = adjustForScroll;
  }

  @Override
  public void setScrollSupport(ScrollSupport support) {
    this.scrollSupport = support;
  }

  public void setStyleFloat(Style.Float styleFloat) {
    this.styleFloat = styleFloat;
  }

  @Override
  protected void doLayout() {
    Size size = getContainerTarget().getStyleSize();

    int w = size.getWidth() - (adjustForScroll ? XDOM.getScrollBarWidth() : 0);
    w -= getContainerTarget().getFrameWidth(Side.LEFT, Side.RIGHT);
    int pw = w;

    int count = getWidgetCount();

    // some columns can be percentages while others are fixed
    // so we need to make 2 passes
    for (int i = 0; i < count; i++) {
      Widget widget = getWidget(i);

      CssFloatData layoutData = null;
      Object d = widget.getLayoutData();
      if (d != null && d instanceof CssFloatData) {
        layoutData = (CssFloatData) d;
      } else {
        layoutData = new CssFloatData();
      }

      if (layoutData.getSize() > 1) {
        pw -= layoutData.getSize();
      }

      pw -= getSideMargins(widget);
    }

    pw = pw < 0 ? 0 : pw;

    for (int i = 0; i < count; i++) {
      Widget widget = getWidget(i);

      CssFloatData layoutData = null;
      Object d = widget.getLayoutData();
      if (d != null && d instanceof CssFloatData) {
        layoutData = (CssFloatData) d;
      } else {
        layoutData = new CssFloatData();
      }

      int width = -1;
      double s = layoutData.getSize();
      if (s > 0 && s <= 1) {
        width = (int) (s * pw);
      } else {
        width = (int) s;
      }

      applyLayout(widget, width, -1);

    }
  }
  
  @Override
  protected XElement getContainerTarget() {
    return appearance.getContainerTarget(getElement());
  }

  @Override
  protected void onInsert(int index, Widget child) {
    super.onInsert(index, child);
    child.getElement().getStyle().setFloat(styleFloat);
    appearance.onInsert(child);
  }

  @Override
  protected void onRemove(Widget child) {
    super.onRemove(child);
    appearance.onRemove(child);
  }

}
