package com.sencha.gxt.widget.core.client.form;

import java.util.List;

import com.google.gwt.editor.client.EditorError;
import com.google.gwt.event.dom.client.BlurEvent;
import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ChangeHandler;
import com.google.gwt.event.dom.client.HasChangeHandlers;
import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.event.shared.HandlerRegistration;
import com.sencha.gxt.cell.core.client.form.CheckBoxCell;

/**
 * Simple checkbox field. {@link ValueChangeEvent}s are fired when the checkbox
 * state is changed by the user, instead of waiting for a {@link BlurEvent}.
 */
public class CheckBox extends Field<Boolean> implements HasChangeHandlers {

  public CheckBox() {
    this(new CheckBoxCell());
  }

  public CheckBox(CheckBoxCell cell) {
    super(cell);
    
    setValue(false, false);
  }
  
  @Override
  public HandlerRegistration addChangeHandler(ChangeHandler handler) {
    return addDomHandler(handler, ChangeEvent.getType());
  }
  
  @Override
  public void clearInvalid() {
    // do nothing
  }

  public String getBoxLabel() {
    return getCell().getBoxLabel();
  }
  
  @Override
  public CheckBoxCell getCell() {
    return (CheckBoxCell) super.getCell();
  }

  public void setBoxLabel(String boxLabel) {
    getCell().setBoxLabel(getElement(), boxLabel);
  }

  @Override
  public void setReadOnly(boolean readOnly) {
    super.setReadOnly(readOnly);
    getCell().getInputElement(getElement()).setReadOnly(readOnly);
  }

  @Override
  public void setValue(Boolean value) {
    // TODO if we support a default value, replace this with that
    if (value == null) {
      value = false;
    }
    setValue(value, true);
  }

  @Override
  protected void markInvalid(List<EditorError> msg) {
    // do nothing
  }

}
