package com.sencha.gxt.widget.core.client.grid;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Stack;

import com.google.gwt.cell.client.Cell;
import com.google.gwt.cell.client.Cell.Context;
import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.JavaScriptObject;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.EventTarget;
import com.google.gwt.dom.client.Node;
import com.google.gwt.dom.client.NodeList;
import com.google.gwt.dom.client.Style.Overflow;
import com.google.gwt.dom.client.TableRowElement;
import com.google.gwt.dom.client.TableSectionElement;
import com.google.gwt.event.logical.shared.SelectionEvent;
import com.google.gwt.event.logical.shared.SelectionHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.google.gwt.safehtml.shared.SafeHtml;
import com.google.gwt.safehtml.shared.SafeHtmlBuilder;
import com.google.gwt.safehtml.shared.SafeHtmlUtils;
import com.google.gwt.user.client.DOM;
import com.google.gwt.user.client.Event;
import com.google.gwt.user.client.ui.HasHorizontalAlignment.HorizontalAlignmentConstant;
import com.google.gwt.user.client.ui.Widget;
import com.sencha.gxt.access.client.Access;
import com.sencha.gxt.core.client.GXT;
import com.sencha.gxt.core.client.Style;
import com.sencha.gxt.core.client.ValueProvider;
import com.sencha.gxt.core.client.dom.DomHelper;
import com.sencha.gxt.core.client.dom.XDOM;
import com.sencha.gxt.core.client.dom.XElement;
import com.sencha.gxt.core.client.resources.CommonStyles;
import com.sencha.gxt.core.client.util.DelayedTask;
import com.sencha.gxt.core.client.util.Point;
import com.sencha.gxt.core.client.util.Size;
import com.sencha.gxt.core.client.util.Util;
import com.sencha.gxt.data.shared.ListStore;
import com.sencha.gxt.data.shared.SortDir;
import com.sencha.gxt.data.shared.SortInfo;
import com.sencha.gxt.data.shared.SortInfoBean;
import com.sencha.gxt.data.shared.Store.StoreSortInfo;
import com.sencha.gxt.data.shared.event.StoreAddEvent;
import com.sencha.gxt.data.shared.event.StoreClearEvent;
import com.sencha.gxt.data.shared.event.StoreDataChangeEvent;
import com.sencha.gxt.data.shared.event.StoreFilterEvent;
import com.sencha.gxt.data.shared.event.StoreHandlers;
import com.sencha.gxt.data.shared.event.StoreRecordChangeEvent;
import com.sencha.gxt.data.shared.event.StoreRemoveEvent;
import com.sencha.gxt.data.shared.event.StoreSortEvent;
import com.sencha.gxt.data.shared.event.StoreUpdateEvent;
import com.sencha.gxt.messages.client.DefaultMessages;
import com.sencha.gxt.widget.core.client.Component;
import com.sencha.gxt.widget.core.client.ComponentHelper;
import com.sencha.gxt.widget.core.client.event.BodyScrollEvent;
import com.sencha.gxt.widget.core.client.event.CheckChangeEvent;
import com.sencha.gxt.widget.core.client.event.CheckChangeEvent.CheckChangeHandler;
import com.sencha.gxt.widget.core.client.event.ColumnMoveEvent;
import com.sencha.gxt.widget.core.client.event.ColumnWidthChangeEvent;
import com.sencha.gxt.widget.core.client.event.RefreshEvent;
import com.sencha.gxt.widget.core.client.event.SortChangeEvent;
import com.sencha.gxt.widget.core.client.menu.CheckMenuItem;
import com.sencha.gxt.widget.core.client.menu.Item;
import com.sencha.gxt.widget.core.client.menu.Menu;
import com.sencha.gxt.widget.core.client.menu.MenuItem;

/**
 * This class encapsulates the user interface of an {@link Grid}. Methods of
 * this class may be used to access user interface elements to enable special
 * display effects. Do not change the DOM structure of the user interface. </p>
 * <p />
 * This class does not provide ways to manipulate the underlying data. The data
 * model of a Grid is held in an {@link ListStore}.
 */
public class GridView<M> {

  protected int activeHdIndex;

  protected boolean autoFill;
  protected int borderWidth = 2;
  protected ColumnModel<M> cm;
  protected ColumnModelHandlers columnListener;
  protected boolean deferEmptyText;
  protected ListStore<M> ds;

  // elements
  protected XElement el, mainWrap, mainHd, innerHd, scroller, mainBody, focusEl;
  protected String emptyText = "&nbsp;";
  protected boolean enableHdMenu = true;

  // config
  protected boolean enableRowBody;
  protected boolean focusEnabled = true;
  protected ColumnFooter<M> footer;

  protected boolean forceFit;
  protected Grid<M> grid;
  protected ColumnHeader<M> header;
  protected int headerColumnIndex;
  protected boolean headerDisabled;
  protected int lastViewWidth;
  protected StoreHandlers<M> listener;
  protected Element overRow;
  protected boolean preventScrollToTopOnRefresh;
  protected int scrollOffset = XDOM.getScrollBarWidth();

  protected boolean selectable = false;
  protected SortInfo sortState;
  protected int splitterWidth = 5;
  protected StoreSortInfo<M> storeSortInfo;
  protected boolean userResized;
  // we first render grid with a vbar, and remove as needed
  protected boolean vbar = true;
  protected String unselectable = CommonStyles.get().unselectableSingle();

  protected GridViewConfig<M> viewConfig;
  private DelayedTask addTask = new DelayedTask() {

    @Override
    public void onExecute() {
      calculateVBar(false);
      refreshFooterData();

    }

  };

  private boolean adjustForHScroll = true;
  private ColumnConfig<M, ?> autoExpandColumn;
  private int autoExpandMax = 500;
  private int autoExpandMin = 25;
  private String cellSelector = "td.x-grid3-cell";
  private int cellSelectorDepth = 15;
  private HandlerRegistration cmHandlerRegistration;
  private DelayedTask removeTask = new DelayedTask() {

    @Override
    public void onExecute() {
      calculateVBar(false);
      applyEmptyText();
      refreshFooterData();
      processRows(0, false);

    }

  };

  private String rowSelector = "div.x-grid3-row";
  private int rowSelectorDepth = 20;
  private boolean showDirtyCells = true;
  private HandlerRegistration storeHandlerRegistration;

  /**
   * Ensured the current row and column is visible.
   * 
   * @param row the row index
   * @param col the column index
   * @param hscroll true to scroll horizontally if needed
   * @return the calculated point
   */
  public Point ensureVisible(int row, int col, boolean hscroll) {
    if (grid == null || !grid.isViewReady() || row < 0 || row > ds.size()) {
      return null;
    }

    if (col == -1) {
      col = 0;
    }

    Element rowEl = getRow(row);
    Element cellEl = null;
    if (!(!hscroll && col == 0)) {
      while (cm.isHidden(col)) {
        col++;
      }
      cellEl = getCell(row, col);

    }

    if (rowEl == null) {
      return null;
    }

    Element c = scroller;

    int ctop = 0;
    Element p = rowEl, stope = el;
    while (p != null && p != stope) {
      ctop += p.getOffsetTop();
      p = p.getOffsetParent();
    }
    ctop -= mainHd.getOffsetHeight();

    int cbot = ctop + rowEl.getOffsetHeight();

    int ch = c.getOffsetHeight();
    int stop = c.getScrollTop();
    int sbot = stop + ch;

    if (ctop < stop) {
      c.setScrollTop(ctop);
    } else if (cbot > sbot) {
      if (hscroll && (cm.getTotalWidth() > scroller.getWidth(false) - scrollOffset)) {
        cbot += scrollOffset;
      }
      c.setScrollTop(cbot -= ch);
    }

    if (hscroll && cellEl != null) {
      int cleft = cellEl.getOffsetLeft();
      int cright = cleft + cellEl.getOffsetWidth();
      int sleft = c.getScrollLeft();
      int sright = sleft + c.getOffsetWidth();
      if (cleft < sleft) {
        c.setScrollLeft(cleft);
      } else if (cright > sright) {
        c.setScrollLeft(cright - scroller.getComputedWidth());
      }
    }

    return cellEl != null ? fly(cellEl).getXY() : new Point(c.getScrollLeft(), fly(rowEl).getY());
  }

  /**
   * Returns the cell.
   * 
   * @param elem the cell element or a child element
   * @return the cell element
   */
  public Element findCell(Element elem) {
    if (elem == null) {
      return null;
    }
    return fly(elem).findParentElement(cellSelector, cellSelectorDepth);
  }

  /**
   * Returns the cell index.
   * 
   * @param elem the cell or child element
   * @param requiredStyle an optional required style name
   * @return the cell index or -1 if not found
   */
  public int findCellIndex(Element elem, String requiredStyle) {
    Element cell = findCell(elem);
    if (cell != null && (requiredStyle == null || fly(cell).hasClassName(requiredStyle))) {
      return Util.parseInt(cell.getAttribute("gridColumnIndex"), -1);
    }
    return -1;
  }

  /**
   * Returns the row element.
   * 
   * @param el the row element or any child element
   * @return the matching row element
   */
  public Element findRow(Element el) {
    if (el == null) {
      return null;
    }
    return fly(el).findParentElement(rowSelector, rowSelectorDepth);
  }

  /**
   * Returns the row index.
   * 
   * @param elem the row or child of the row element
   * @return the index
   */
  public int findRowIndex(Element elem) {
    Element r = findRow(elem);
    if (r != null) {
      return r.getPropertyInt("rowIndex");
    }
    return -1;
  }

  /**
   * Focus the cell and scrolls into view.
   * 
   * @param rowIndex the row index
   * @param colIndex the column index
   * @param hscroll true to scroll horizontally
   */
  public void focusCell(int rowIndex, int colIndex, boolean hscroll) {
    Point xy = ensureVisible(rowIndex, colIndex, hscroll);
    if (xy != null) {
      focusEl.setXY(xy);
      if (focusEnabled) {
        focusGrid();
      }
    }
  }

  /**
   * Focus the row and scrolls into view.
   * 
   * @param rowIndex the row index
   */
  public void focusRow(int rowIndex) {
    focusCell(rowIndex, 0, true);
  }

  /**
   * Returns the auto expand column id.
   * 
   * @return the auto expand column id
   */
  public ColumnConfig<M, ?> getAutoExpandColumn() {
    return autoExpandColumn;
  }

  /**
   * Returns the auto expand maximum width.
   * 
   * @return the max width in pixels
   */
  public int getAutoExpandMax() {
    return autoExpandMax;
  }

  /**
   * Returns the auto expand minimum width.
   * 
   * @return the minimum width in pixels
   */
  public int getAutoExpandMin() {
    return autoExpandMin;
  }

  /**
   * Returns the grid's body element.
   * 
   * @return the body element
   */
  public XElement getBody() {
    return mainBody;
  }

  /**
   * Returns the grid's &lt;TD> HtmlElement at the specified coordinates.
   * 
   * @param row the row index in which to find the cell
   * @param col the column index of the cell
   * @return the &lt;TD> at the specified coordinates
   */
  public Element getCell(int row, int col) {
    // ROW DIV TABLE TR TD
    Element rowEl = getRow(row);
    if (rowEl == null || !rowEl.hasChildNodes() || col < 0) {
      return null;
    } else {
      return (Element) rowEl.getFirstChild().getFirstChild().getFirstChild().getChildNodes().getItem(col);
    }
  }

  /**
   * Returns the cell selector depth.
   * 
   * @return the cell selector depth
   */
  public int getCellSelectorDepth() {
    return cellSelectorDepth;
  }

  /**
   * Returns the editor parent element.
   * 
   * @return the editor element
   */
  public Element getEditorParent() {
    return scroller;
  }

  /**
   * Returns the empty text.
   * 
   * @return the empty text
   */
  public String getEmptyText() {
    return emptyText;
  }

  /**
   * Returns the grid's column header.
   * 
   * @return the header
   */
  public ColumnHeader<M> getHeader() {
    return header;
  }

  /**
   * Returns the &lt;TD> HtmlElement which represents the Grid's header cell for
   * the specified column index.
   * 
   * @param index the column index
   * @return the &lt;TD> element.
   */
  public Element getHeaderCell(int index) {
    return mainHd.getElementsByTagName("td").getItem(index);
  }

  public Element getInnerCellElement(int row, int col) {
    Element cellElement = getCell(row, col);
    if (cellElement != null) {
      return cellElement.getFirstChildElement();
    }
    return null;
  }

  /**
   * Return the &lt;TR> HtmlElement which represents a Grid row for the
   * specified index.
   * 
   * @param row the row index
   * @return the &lt;TR> element
   */
  public Element getRow(int row) {
    if (row < 0) {
      return null;
    }
    return getRows().getItem(row);
  }

  /**
   * Return the &lt;TR> HtmlElement which represents a Grid row for the
   * specified model.
   * 
   * @param m the model
   * @return the &lt;TR> element
   */
  public Element getRow(M m) {
    return getRow(ds.indexOf(m));
  }

  /**
   * Returns the row selector depth.
   * 
   * @return the row selector depth
   */
  public int getRowSelectorDepth() {
    return rowSelectorDepth;
  }

  /**
   * Returns the scroll element.
   * 
   * @return the scroll element
   */
  public XElement getScroller() {
    return scroller;
  }

  /**
   * Returns the current scroll state.
   * 
   * @return the scroll state
   */
  public Point getScrollState() {
    return new Point(scroller.getScrollLeft(), scroller.getScrollTop());
  }

  /**
   * Returns the view config.
   * 
   * @return the view config
   */
  public GridViewConfig<M> getViewConfig() {
    return viewConfig;
  }

  /**
   * Returns true if the grid width will be adjusted based on visibility of
   * horizontal scroll bar.
   * 
   * @return true if adjusting
   */
  public boolean isAdjustForHScroll() {
    return adjustForHScroll;
  }

  /**
   * Returns true if auto fill is enabled.
   * 
   * @return true for auto fill
   */
  public boolean isAutoFill() {
    return autoFill;
  }

  /**
   * Returns true if force fit is enabled.
   * 
   * @return true for force fit
   */
  public boolean isForceFit() {
    return forceFit;
  }

  public boolean isSelectableTarget(Element target) {
    if (target == null) {
      return false;
    }

    String tag = target.getTagName();
    if ("input".equalsIgnoreCase(tag) || "textarea".equalsIgnoreCase(tag)) {
      return false;
    }

    int colIndex = findCellIndex(target, null);
    Element cellParent = getInnerCellElement(findRowIndex(target), colIndex);

    com.google.gwt.cell.client.Cell<?> cell = grid.getColumnModel().getCell(colIndex);
    if (cell != null && cellParent != null && cellParent.isOrHasChild(target) && cell.handlesSelection()) {
      return false;
    }
    return true;
  }

  /**
   * Returns true if dirty cell markers are enabled.
   * 
   * @return true of dirty cell markers
   */
  public boolean isShowDirtyCells() {
    return showDirtyCells;
  }

  /**
   * Returns true if sorting is enabled.
   * 
   * @return true for sorting
   */
  public boolean isSortingEnabled() {
    return !headerDisabled;
  }

  public void layout() {
    layout(false);
  }

  /**
   * Rebuilds the grid using its current configuration and data.
   * 
   * @param headerToo true to refresh the header
   */
  public void refresh(boolean headerToo) {
    if (grid != null && grid.isViewReady()) {

      if (!preventScrollToTopOnRefresh) {
        scrollToTop();
      }
      mainBody.setInnerHTML(renderRows(0, -1).asString());
      if (headerToo) {
        sortState = null;
        header.release();

        newColumnHeader();
        renderHeader();
        if (grid.isAttached()) {
          ComponentHelper.doAttach(header);
        }
        header.setEnableColumnResizing(grid.isColumnResize());
        header.setEnableColumnReorder(grid.isColumnReordering());
      }
      processRows(0, true);

      if (footer != null) {
        ComponentHelper.doDetach(footer);
        footer.getElement().removeFromParent();
      }
      if (cm.getAggregationRows().size() > 0) {
        footer = new ColumnFooter<M>(grid, cm);
        renderFooter();
        if (grid.isAttached()) {
          ComponentHelper.doAttach(footer);
        }
      }

      calculateVBar(true);

      updateHeaderSortState();

      applyEmptyText();
      constrainFocusElement();

      grid.fireEvent(new RefreshEvent());
    }
  }

  /**
   * Scrolls the grid to the top.
   */
  public void scrollToTop() {
    scroller.setScrollTop(0);
    scroller.setScrollLeft(0);
  }

  /**
   * True to adjust the grid width when the horizontal scrollbar is hidden and
   * visible (defaults to true).
   * 
   * @param adjustForHScroll true to adjust for horizontal scroll bar
   */
  public void setAdjustForHScroll(boolean adjustForHScroll) {
    this.adjustForHScroll = adjustForHScroll;
  }

  /**
   * The id of a column in this grid that should expand to fill unused space
   * (pre-render). This id can not be 0.
   * 
   * @param autoExpandColumn the auto expand column
   */
  public void setAutoExpandColumn(ColumnConfig<M, ?> autoExpandColumn) {
    this.autoExpandColumn = autoExpandColumn;
  }

  /**
   * The maximum width the autoExpandColumn can have (if enabled) (defaults to
   * 500, pre-render).
   * 
   * @param autoExpandMax the auto expand max
   */
  public void setAutoExpandMax(int autoExpandMax) {
    this.autoExpandMax = autoExpandMax;
  }

  /**
   * The minimum width the autoExpandColumn can have (if enabled)(pre-render).
   * 
   * @param autoExpandMin the auto expand min width
   */
  public void setAutoExpandMin(int autoExpandMin) {
    this.autoExpandMin = autoExpandMin;
  }

  /**
   * True to auto expand the columns to fit the grid <b>when the grid is
   * created</b>.
   * 
   * @param autoFill true to expand
   */
  public void setAutoFill(boolean autoFill) {
    this.autoFill = autoFill;
  }

  /**
   * The number of levels to search for cells in event delegation (defaults to
   * 4).
   * 
   * @param cellSelectorDepth the cell selector depth
   */
  public void setCellSelectorDepth(int cellSelectorDepth) {
    this.cellSelectorDepth = cellSelectorDepth;
  }

  /**
   * Default text to display in the grid body when no rows are available
   * (defaults to '').
   * 
   * @param emptyText the empty text
   */
  public void setEmptyText(String emptyText) {
    this.emptyText = emptyText;
  }

  /**
   * True to auto expand/contract the size of the columns to fit the grid width
   * and prevent horizontal scrolling.
   * 
   * @param forceFit true to force fit
   */
  public void setForceFit(boolean forceFit) {
    this.forceFit = forceFit;
  }

  /**
   * The number of levels to search for rows in event delegation (defaults to
   * 10).
   * 
   * @param rowSelectorDepth the row selector depth
   */
  public void setRowSelectorDepth(int rowSelectorDepth) {
    this.rowSelectorDepth = rowSelectorDepth;
  }

  /**
   * True to display a red triangle in the upper left corner of any cells which
   * are "dirty" as defined by any existing records in the data store (defaults
   * to true).
   * 
   * @param showDirtyCells true to display the dirty flag
   */
  public void setShowDirtyCells(boolean showDirtyCells) {
    this.showDirtyCells = showDirtyCells;
  }

  /**
   * True to allow column sorting when the user clicks a column (defaults to
   * true).
   * 
   * @param sortable true for sortable columns
   */
  public void setSortingEnabled(boolean sortable) {
    this.headerDisabled = !sortable;
  }

  /**
   * Sets the view config.
   * 
   * @param viewConfig the view config
   */
  public void setViewConfig(GridViewConfig<M> viewConfig) {
    this.viewConfig = viewConfig;
  }

  protected void addRowStyle(Element elem, String style) {
    if (elem != null) {
      fly(elem).addClassName(style);
    }
  }

  protected void afterRender() {
    mainBody.setInnerHTML(renderRows(0, -1).asString());
    processRows(0, true);

    if (footer != null && grid.getLazyRowRender() > 0) {
      footer.refresh();
    }

    int sh = scroller.getComputedHeight();
    int dh = mainBody.getComputedHeight();
    boolean vbar = dh < sh;
    if (vbar) {
      this.vbar = !vbar;
      lastViewWidth = -1;
      layout();
    }

    applyEmptyText();

  }

  protected void applyEmptyText() {
    if (emptyText == null) {
      emptyText = "&nbsp;";
    }
    if (!hasRows()) {
      mainBody.setInnerHTML("<div class='x-grid-empty'>" + emptyText + "</div>");
    }
    syncHScroll();
  }

  protected void autoExpand(boolean preventUpdate) {
    if (!userResized && getAutoExpandColumn() != null) {
      int tw = cm.getTotalWidth(false);
      int aw = grid.getOffsetWidth(true) - getScrollAdjust();
      if (tw != aw) {
        int ci = cm.indexOf(getAutoExpandColumn());
        assert ci != Style.DEFAULT : "auto expand column not found";
        if (cm.isHidden(ci)) {
          return;
        }
        int currentWidth = cm.getColumnWidth(ci);
        int cw = Math.min(Math.max(((aw - tw) + currentWidth), getAutoExpandMin()), getAutoExpandMax());
        if (cw != currentWidth) {
          cm.setColumnWidth(ci, cw, true);

          if (!preventUpdate) {
            updateColumnWidth(ci, cw);
          }
        }
      }
    }
  }

  protected void calculateVBar(boolean force) {
    if (force) {
      resize();
    }
    int sh = scroller.getComputedHeight();
    int dh = mainBody.getComputedHeight();
    boolean vbar = dh > sh;
    if (force || this.vbar != vbar) {
      this.vbar = vbar;
      lastViewWidth = -1;
      layout(true);
    }
  }

  protected Menu createContextMenu(final int colIndex) {
    final Menu menu = new Menu();

    if (cm.isSortable(colIndex)) {
      MenuItem item = new MenuItem();
      item.setText(DefaultMessages.getMessages().gridView_sortAscText());
      item.setIcon(grid.getAppearance().sortAcendingIcon());
      item.addSelectionHandler(new SelectionHandler<Item>() {

        @Override
        public void onSelection(SelectionEvent<Item> event) {
          doSort(colIndex, SortDir.ASC);

        }
      });
      menu.add(item);

      item = new MenuItem();
      item.setText(DefaultMessages.getMessages().gridView_sortDescText());
      item.setIcon(grid.getAppearance().sortDescendingIcon());
      item.addSelectionHandler(new SelectionHandler<Item>() {

        @Override
        public void onSelection(SelectionEvent<Item> event) {
          doSort(colIndex, SortDir.DESC);

        }
      });
      menu.add(item);
    }

    MenuItem columns = new MenuItem();
    columns.setText(DefaultMessages.getMessages().gridView_columnsText());
    columns.setIcon(grid.getAppearance().columnsIcon());
    columns.setData("gxt-columns", "true");

    final Menu columnMenu = new Menu();

    int cols = cm.getColumnCount();
    for (int i = 0; i < cols; i++) {
      if (shouldNotCount(i, false)) {
        continue;
      }
      final int fcol = i;
      final CheckMenuItem check = new CheckMenuItem();
      check.setHideOnClick(false);
      check.setText(cm.getColumnHeader(i).asString());
      check.setChecked(!cm.isHidden(i));
      check.addCheckChangeHandler(new CheckChangeHandler<CheckMenuItem>() {

        @Override
        public void onCheckChange(CheckChangeEvent<CheckMenuItem> event) {
          cm.setHidden(fcol, !cm.isHidden(fcol));
          restrictMenu(columnMenu);

        }
      });
      columnMenu.add(check);
    }

    restrictMenu(columnMenu);
    columns.setEnabled(columnMenu.getWidgetCount() > 0);
    columns.setSubMenu(columnMenu);
    menu.add(columns);
    return menu;
  }

  protected void doAttach() {
    ComponentHelper.doAttach(header);
    ComponentHelper.doAttach(footer);
    resize();
  }

  protected void doDetach() {
    ComponentHelper.doDetach(header);
    ComponentHelper.doDetach(footer);
  }

  protected SafeHtml doRender(List<ColumnData> cs, List<M> rows, int startRow) {
    int colCount = cm.getColumnCount();
    int last = colCount - 1;
    String tstyle = "width:" + getTotalWidth() + "px;";

    SafeHtmlBuilder buf = new SafeHtmlBuilder();

    for (int j = 0; j < rows.size(); j++) {
      M model = rows.get(j);

      ListStore<M>.Record r = ds.hasRecord(model) ? ds.getRecord(model) : null;

      int rowBodyColSpanCount = colCount;
      if (enableRowBody) {
        // if (grid.getSelectionModel() instanceof CheckBoxSelectionModel<?>) {
        // CheckBoxSelectionModel<?> sm = (CheckBoxSelectionModel<?>)
        // grid.getSelectionModel();
        // if (cm.getColumnById(sm.getColumn().getId()) != null) {
        // rowBodyColSpanCount--;
        // }
        // }
        for (ColumnConfig<M, ?> c : cm.getColumns()) {
          if (c instanceof RowExpander) {
            rowBodyColSpanCount--;
          }
        }
      }
      int rowIndex = (j + startRow);
      buf.append(SafeHtmlUtils.fromTrustedString("<div rowIndex=\""));
      buf.append(rowIndex);
      if (Access.ariaIsEnabled()) {
        buf.append(SafeHtmlUtils.fromTrustedString("\" role=\"row\" aria-level=\"2\" class=\"x-grid3-row "));
      } else {
        buf.append(SafeHtmlUtils.fromTrustedString("\" class=\"x-grid3-row "));
      }

      if (grid.isStripeRows() && ((rowIndex + 1) % 2 == 0)) {
        buf.append(SafeHtmlUtils.fromTrustedString(" x-grid3-row-alt"));
      }
      if (!selectable) {
        buf.append(SafeHtmlUtils.fromTrustedString(" " + unselectable));
      }

      if (showDirtyCells && r != null && r.isDirty()) {
        buf.append(SafeHtmlUtils.fromTrustedString(" x-grid3-dirty-row"));
      }
      if (viewConfig != null) {
        buf.append(SafeHtmlUtils.fromTrustedString(" "));
        buf.append(SafeHtmlUtils.fromTrustedString(viewConfig.getRowStyle(model, rowIndex)));
      }
      buf.append(SafeHtmlUtils.fromTrustedString("\" style=\""));
      buf.append(SafeHtmlUtils.fromTrustedString(tstyle));
      buf.append(SafeHtmlUtils.fromTrustedString("\" id=\""));
      buf.append(SafeHtmlUtils.fromTrustedString(grid.getId()));
      buf.append(SafeHtmlUtils.fromTrustedString("_"));
      buf.append(SafeHtmlUtils.fromTrustedString(ds.getKeyProvider() != null ? ds.getKeyProvider().getKey(model)
          : XDOM.getUniqueId()));
      buf.append(SafeHtmlUtils.fromTrustedString("\" unselectable=\""));
      buf.append(SafeHtmlUtils.fromTrustedString(selectable ? "off" : "on"));
      buf.append(SafeHtmlUtils.fromTrustedString("\"><table class=x-grid3-row-table role=presentation border=0 cellspacing=0 cellpadding=0 style=\""));

      buf.append(SafeHtmlUtils.fromTrustedString(tstyle));
      buf.append(SafeHtmlUtils.fromTrustedString("\"><tbody role=presentation><tr role=presentation>"));
      for (int i = 0; i < colCount; i++) {
        SafeHtml rv = getRenderedValue(rowIndex, i, model, r);
        String role = "gridcell";
        ColumnConfig<M, ?> cc = cm.getColumn(i);
        ColumnData cd = cs.get(i);
        if (Access.ariaIsEnabled()) {

          if (cc.isRowHeader()) {
            role = "rowheader";
          }
        }

        String cellAttr = cd.cellAttr != null ? cd.cellAttr : "";

        buf.append(SafeHtmlUtils.fromTrustedString("<td gridColumnIndex=\"" + i + "\" id=\"" + XDOM.getUniqueId()
            + "\" role=\"" + role + "\" class=\"x-grid3-col x-grid3-cell "));
        buf.append(SafeHtmlUtils.fromTrustedString(i == 0 ? "x-grid-cell-first " : (i == last ? "x-grid3-cell-last "
            : "")));

        String id = cc.getColumnClassName();
        if (id != null && !id.equals("")) {
          buf.appendHtmlConstant(" x-grid3-td-" + id);
        }

        if (showDirtyCells && r != null && r.getChange(cc.getValueProvider()) != null) {
          buf.append(SafeHtmlUtils.fromTrustedString(" x-grid3-dirty-cell"));
        }

        if (viewConfig != null) {
          buf.append(SafeHtmlUtils.fromTrustedString(" "));
          buf.append(SafeHtmlUtils.fromTrustedString(viewConfig.getColStyle(model, cm.getValueProvider(i), rowIndex, i)));
        }

        buf.append(SafeHtmlUtils.fromTrustedString("\" style=\""));
        buf.append(SafeHtmlUtils.fromTrustedString(cd.style));
        buf.appendHtmlConstant("\" ");
        buf.append(SafeHtmlUtils.fromTrustedString(cellAttr));
        buf.append(SafeHtmlUtils.fromTrustedString("><div unselectable=\""));
        buf.append(SafeHtmlUtils.fromTrustedString(selectable ? "off" : "on"));
        buf.append(SafeHtmlUtils.fromTrustedString("\" class=\"x-grid3-cell-inner"));
        buf.append(SafeHtmlUtils.fromTrustedString("\">"));
        buf.append(rv);
        buf.append(SafeHtmlUtils.fromTrustedString("</div></td>"));
      }

      buf.append(SafeHtmlUtils.fromTrustedString("</tr>"));
      if (enableRowBody) {
        buf.append(SafeHtmlUtils.fromTrustedString("<tr class=x-grid3-row-body-tr style=\"\"><td colspan="));
        buf.append(rowBodyColSpanCount);
        buf.append(SafeHtmlUtils.fromTrustedString(" class=x-grid3-body-cell><div class=x-grid3-row-body>${body}</div></td></tr>"));
      }
      buf.append(SafeHtmlUtils.fromTrustedString("</tbody></table></div>"));
    }

    return buf.toSafeHtml();
  }

  /**
   * Defaults to assume one sort at a time.
   * 
   * @param colIndex the column to sort
   * @param sortDir the sort direction
   */
  protected <N> void doSort(int colIndex, SortDir sortDir) {
    ColumnConfig<M, ?> column = cm.getColumn(colIndex);
    if (!isRemoteSort()) {
      ds.clearSortInfo();

      // These casts can fail, but in dev mode the exception will be caught by
      // the
      // try/catch, unless there are no items in the Store
      @SuppressWarnings({"unchecked", "rawtypes"})
      ValueProvider<M, Comparable> vp = (ValueProvider) column.getValueProvider();
      @SuppressWarnings("unchecked")
      StoreSortInfo<M> s = new StoreSortInfo<M>(vp, sortDir);

      if (sortDir == null && storeSortInfo != null && storeSortInfo.getValueProvider() == vp) {
        s.setDirection(storeSortInfo.getDirection() == SortDir.ASC ? SortDir.DESC : SortDir.ASC);
      } else if (sortDir == null) {
        s.setDirection(SortDir.ASC);
      }

      if (GWT.isProdMode()) {
        ds.addSortInfo(s);
      } else {
        try {
          // addSortInfo will apply its sort when called, which might trigger an
          // exception if the column passed in's data isn't Comparable
          ds.addSortInfo(s);
        } catch (ClassCastException ex) {
          GWT.log("Column can't be sorted " + column.getValueProvider().getPath() + " is not Comparable. ", ex);
          throw ex;
        }
      }
    } else {
      ValueProvider<M, ?> vp = column.getValueProvider();

      SortInfoBean bean = new SortInfoBean(vp, sortDir);

      if (sortDir == null && sortState != null && vp.getPath().equals(sortState.getSortField())) {
        bean.setSortDir(sortState.getSortDir() == SortDir.ASC ? SortDir.DESC : SortDir.ASC);

      } else if (sortDir == null) {
        bean.setSortDir(SortDir.ASC);
      }

      grid.getLoader().clearSortInfo();
      grid.getLoader().addSortInfo(bean);
      grid.getLoader().load();
    }
  }

  protected void fitColumns(boolean preventRefresh, boolean onlyExpand, int omitColumn) {
    int tw = cm.getTotalWidth(false);
    int aw = grid.getElement().getWidth(true) - getScrollAdjust();
    if (aw <= 0) {
      aw = grid.getElement().getComputedWidth();
    }

    if (aw < 20 || aw > 2000) { // not initialized, so don't screw up the
      // default widths
      return;
    }

    int extra = (int) aw - tw;

    if (extra == 0) {
      return;
    }

    int colCount = cm.getColumnCount();
    Stack<Integer> cols = new Stack<Integer>();
    int width = 0;
    int w;

    for (int i = 0; i < colCount; i++) {
      if (!cm.isHidden(i) && !cm.isFixed(i) && i != omitColumn) {
        w = cm.getColumnWidth(i);
        cols.push(i);
        cols.push(w);
        width += w;
      }
    }

    double frac = ((double) (aw - cm.getTotalWidth())) / width;
    while (cols.size() > 0) {
      w = cols.pop();
      int i = cols.pop();
      int ww = Math.max(grid.getMinColumnWidth(), (int) Math.floor(w + w * frac));
      cm.setColumnWidth(i, ww, true);
    }

    int adj = aw - cm.getTotalWidth();
    if (adj != 0) {
      int ac = cm.getColumnCount() - 1;
      while ((ac == omitColumn || cm.isHidden(ac) || cm.isFixed(ac)) && ac >= 0) {
        ac--;
      }
      if (ac > 0) {
        w = cm.getColumnWidth(ac);
        w += adj;

        cm.setColumnWidth(ac, w, true);
      }
    }

    if (!preventRefresh) {
      updateAllColumnWidths();
    }
  }

  protected XElement fly(Element elem) {
    return XElement.as(elem);
  }

  protected void focusGrid() {
    focusEl.focus();
  }

  protected List<ColumnData> getColumnData() {
    int colCount = cm.getColumnCount();
    List<ColumnData> cs = new ArrayList<ColumnData>();
    for (int i = 0; i < colCount; i++) {
      ColumnData data = new ColumnData();
      data.style = getColumnStyle(i, false);

      ColumnConfig<M, ?> c = cm.getColumn(i);
      if (c instanceof RowExpander) {
        data.cellAttr = "rowspan='2'";
      }

      cs.add(data);
    }
    return cs;
  }

  protected String getColumnStyle(int colIndex, boolean isHeader) {
    String style = !isHeader ? cm.getColumnStyle(colIndex) : "";
    if (style == null) style = "";
    int adj = GXT.isWebKit() ? 2 : 0;
    style += "width:" + (getColumnWidth(colIndex) + adj) + "px;";
    if (cm.isHidden(colIndex)) {
      style += "display:none;";
    }
    HorizontalAlignmentConstant align = cm.getColumnAlignment(colIndex);
    if (align != null) {
      style += "text-align:" + align.getTextAlignString() + ";";
    }
    return style;
  }

  protected int getColumnWidth(int col) {
    int w = cm.getColumnWidth(col);
    return (GXT.isBorderBox() ? w : (w - borderWidth > 0 ? w - borderWidth : 0));
  }

  protected int getOffsetWidth() {
    return (getTotalWidth() + getScrollAdjust());
  }

  protected <N> SafeHtml getRenderedValue(int rowIndex, int colIndex, M m, ListStore<M>.Record record) {
    ValueProvider<M, N> valueProvider = cm.getValueProvider(colIndex);
    N val = null;
    if (record != null) {
      val = record.getValue(valueProvider);
    } else {
      val = valueProvider.getValue(m);
    }
    Cell<N> r = cm.getCell(colIndex);
    if (r != null) {
      SafeHtmlBuilder sb = new SafeHtmlBuilder();
      r.render(new Context(rowIndex, colIndex, ds.getKeyProvider().getKey(m)), val, sb);
      return sb.toSafeHtml();
    }

    String text = null;
    if (val != null) {
      text = val.toString();
    }
    return Util.isEmptyString(text) ? SafeHtmlUtils.fromTrustedString("&#160;") : SafeHtmlUtils.fromString(text);
  }

  protected NodeList<Element> getRows() {
    if (!hasRows()) {
      return JavaScriptObject.createArray().cast();
    }
    return mainBody.getChildNodes().cast();
  }

  protected int getScrollAdjust() {
    return adjustForHScroll ? (scroller != null ? (vbar ? scrollOffset + 4 : 2) : scrollOffset) : scrollOffset;
  }

  protected StoreSortInfo<M> getSortState() {
    if (ds.getSortInfo().size() > 0) {
      return ds.getSortInfo().get(0);
    }
    return null;
  }

  protected int getTotalWidth() {
    return cm.getTotalWidth();
  }

  protected void handleComponentEvent(Event ge) {
    switch (ge.getTypeInt()) {
      case Event.ONMOUSEMOVE:
        Element row = null;
        if (Element.is(ge.getEventTarget())) {
          row = findRow(Element.as(ge.getEventTarget()));
        }
        if (overRow != null && row == null) {
          onRowOut(overRow);
        } else if (row != null && overRow != row) {
          if (overRow != null) {
            onRowOut(overRow);
          }
          onRowOver(row);
        }

        break;

      case Event.ONMOUSEOVER:
        EventTarget from = ge.getRelatedEventTarget();
        if (from == null
            || (Element.is(from) && !DOM.isOrHasChild(grid.getElement(),
                (com.google.gwt.user.client.Element) Element.as(from)))) {
          Element r = null;
          if (Element.is(ge.getEventTarget())) {
            r = findRow(Element.as(ge.getEventTarget()));
          }
          if (r != null) {
            onRowOver(r);
          }
        }
        break;
      case Event.ONMOUSEOUT:
        EventTarget to = ge.getRelatedEventTarget();
        if (to == null
            || (Element.is(to) && !DOM.isOrHasChild(grid.getElement(),
                (com.google.gwt.user.client.Element) Element.as(to)))) {
          if (overRow != null) {
            onRowOut(overRow);
          }
        }
        break;
      case Event.ONMOUSEDOWN:
        onMouseDown(ge);
        break;
      case Event.ONSCROLL:
        if (scroller.isOrHasChild(Element.as(ge.getEventTarget()))) {
          syncScroll();
        }
        break;
    }
  }

  protected boolean hasRows() {
    if (mainBody == null) {
      return false;
    }
    Element e = mainBody.getFirstChildElement();
    return e != null && !"x-grid-empty".equals(e.getClassName());
  }

  /**
   * Initializes the view.
   * 
   * @param grid the grid
   */
  protected void init(final Grid<M> grid) {
    this.grid = grid;
    this.cm = grid.getColumnModel();
    selectable = grid.isAllowTextSelection();

    initListeners();

    initData(grid.getStore(), cm);
    initUI(grid);

    newColumnHeader();

    if (cm.getAggregationRows().size() > 0) {
      footer = new ColumnFooter<M>(grid, cm);
    }
  }

  /**
   * Initializes the data.
   * 
   * @param ds the data store
   * @param cm the column model
   */
  protected void initData(ListStore<M> ds, ColumnModel<M> cm) {
    if (storeHandlerRegistration != null) {
      storeHandlerRegistration.removeHandler();
      storeHandlerRegistration = null;
    }
    if (ds != null) {
      storeHandlerRegistration = ds.addStoreHandlers(listener);
    }
    this.ds = ds;

    if (cmHandlerRegistration != null) {
      cmHandlerRegistration.removeHandler();
      cmHandlerRegistration = null;
    }
    if (cm != null) {
      cmHandlerRegistration = cm.addColumnModelHandlers(columnListener);
    }
    this.cm = cm;
  }

  protected void initElements() {
    NodeList<Node> cs = grid.getElement().getFirstChild().getChildNodes();

    el = (XElement) grid.getElement().getFirstChildElement();

    mainWrap = (XElement) cs.getItem(0);
    mainHd = (XElement) mainWrap.getFirstChildElement();

    if (grid.isHideHeaders()) {
      mainHd.setVisible(false);
    }

    innerHd = (XElement) mainHd.getFirstChildElement();

    scroller = (XElement) mainWrap.getChild(1);
    scroller.addEventsSunk(Event.ONSCROLL);

    if (forceFit) {
      scroller.getStyle().setOverflowX(Overflow.HIDDEN);
    }

    mainBody = (XElement) scroller.getFirstChildElement();
    focusEl = (XElement) scroller.getChild(1);
  }

  protected void initListeners() {
    listener = new StoreHandlers<M>() {

      @Override
      public void onAdd(StoreAddEvent<M> event) {
        GridView.this.onAdd(event.getItems(), event.getIndex());

      }

      @Override
      public void onClear(StoreClearEvent<M> event) {
        GridView.this.onClear(event);

      }

      @Override
      public void onDataChange(StoreDataChangeEvent<M> event) {
        GridView.this.onDataChanged(event);

      }

      @Override
      public void onFilter(StoreFilterEvent<M> event) {
        GridView.this.onDataChanged(null);
      }

      @SuppressWarnings("unchecked")
      @Override
      public void onRecordChange(StoreRecordChangeEvent<M> event) {
        GridView.this.onUpdate(ds, Arrays.<M> asList(event.getRecord().getModel()));

      }

      @Override
      public void onRemove(StoreRemoveEvent<M> event) {
        GridView.this.onRemove(event.getItem(), event.getIndex(), false);

      }

      @Override
      public void onSort(StoreSortEvent<M> event) {
        GridView.this.onDataChanged(null);

      }

      @Override
      public void onUpdate(StoreUpdateEvent<M> event) {
        GridView.this.onUpdate(ds, event.getItems());
      }

    };

    columnListener = new ColumnModelHandlers() {
      @Override
      public void onColumnHeaderChange(ColumnHeaderChangeEvent event) {
        GridView.this.onHeaderChange(event.getIndex(), cm.getColumnHeader(event.getIndex()));

      }

      @Override
      public void onColumnHiddenChange(ColumnHiddenChangeEvent event) {
        GridView.this.onHiddenChange(cm, event.getIndex(), cm.isHidden(event.getIndex()));

      }

      @Override
      public void onColumnMove(ColumnMoveEvent event) {
        GridView.this.onColumnMove(event.getIndex());

      }

      @Override
      public void onColumnWidthChange(ColumnWidthChangeEvent event) {
        GridView.this.onColumnWidthChange(event.getIndex(), cm.getColumnWidth(event.getIndex()));

      }
    };
  }

  protected void initUI(final Grid<M> grid) {

  }

  protected void insertRows(int firstRow, int lastRow, boolean isUpdate) {
    Element e = mainBody.getFirstChildElement();
    if (e != null && !hasRows()) {
      mainBody.setInnerHTML("");
    }

    SafeHtml html = renderRows(firstRow, lastRow);
    Element before = getRow(firstRow);

    if (before != null) {
      DomHelper.insertBefore(before, html.asString());
    } else {
      DomHelper.insertHtml("beforeEnd", mainBody, html.asString());
    }

    if (!isUpdate) {
      processRows(firstRow, false);
    }
  }

  protected boolean isRemoteSort() {
    return grid.getLoader() != null && grid.getLoader().isRemoteSort();
  }

  protected void layout(boolean skipResize) {
    if (mainBody == null) {
      return;
    }

    XElement c = grid.getElement();
    Size csize = c.getStyleSize();

    int vw = csize.getWidth();
    if (vw < 10 || csize.getHeight() < 20) {
      return;
    }

    if (!skipResize) {
      resize();
    }

    if (forceFit || autoFill) {
      if (lastViewWidth != vw) {
        fitColumns(false, false, -1);
        header.updateTotalWidth(getOffsetWidth(), getTotalWidth());
        if (footer != null) {
          footer.updateTotalWidth(getOffsetWidth(), getTotalWidth());
        }
        lastViewWidth = vw;
      }
    } else {
      autoExpand(false);
      header.updateTotalWidth(getOffsetWidth(), getTotalWidth());
      if (footer != null) {
        footer.updateTotalWidth(getOffsetWidth(), getTotalWidth());
      }
      syncHeaderScroll();
    }

  }

  protected ColumnHeader<M> newColumnHeader() {
    header = new ColumnHeader<M>(grid, cm) {

      @Override
      protected Menu getContextMenu(int column) {
        return createContextMenu(column);
      }

      @Override
      protected void onColumnSplitterMoved(int colIndex, int width) {
        super.onColumnSplitterMoved(colIndex, width);
        GridView.this.onColumnSplitterMoved(colIndex, width);
      }

      @Override
      protected void onHeaderClick(Event ce, int column) {
        super.onHeaderClick(ce, column);
        GridView.this.onHeaderClick(grid, column);
      }

      @Override
      protected void onKeyDown(Event ce, int index) {
        ce.stopPropagation();
        // if (grid.getSelectionModel() instanceof CellSelectionModel<?>) {
        // CellSelectionModel<?> csm = (CellSelectionModel<?>)
        // grid.getSelectionModel();
        // csm.selectCell(0, index);
        // } else {
        // grid.getSelectionModel().select(0, false);
        // }
      }

    };
    header.setSplitterWidth(splitterWidth);
    header.setMinColumnWidth(grid.getMinColumnWidth());

    return header;
  }

  protected void notifyHide() {
  }

  protected void notifyShow() {
  }

  protected void onAdd(List<M> models, int index) {
    if (grid != null && grid.isViewReady()) {
      insertRows(index, index + (models.size() - 1), false);
      addTask.delay(10);
    }
  }

  protected void onCellDeselect(int row, int col) {
    Element cell = getCell(row, col);
    if (cell != null) {
      fly(cell).removeClassName("x-grid3-cell-selected");
      if (Access.ariaIsEnabled()) {
        cell.setAttribute("aria-selected", "false");
      }
    }
  }

  protected void onCellSelect(int row, int col) {
    Element cell = getCell(row, col);
    if (cell != null) {
      fly(cell).addClassName("x-grid3-cell-selected");
      if (Access.ariaIsEnabled()) {
        cell.setAttribute("aria-selected", "true");
        grid.setAriaState("aria-activedescendant", cell.getId());
      }
    }
  }

  protected void onClear(StoreClearEvent<M> se) {
    refresh(false);
  }

  protected void onClick(Event ce) {

  }

  protected void onColumnMove(int newIndex) {
    boolean pScroll = preventScrollToTopOnRefresh;
    preventScrollToTopOnRefresh = true;
    refresh(true);
    preventScrollToTopOnRefresh = pScroll;
  }

  protected void onColumnSplitterMoved(int colIndex, int width) {
    userResized = true;
    width = Math.max(grid.getMinColumnWidth(), width);
    cm.setColumnWidth(colIndex, width);
  }

  protected void onColumnWidthChange(int column, int width) {
    if (forceFit) {
      fitColumns(false, false, column);
      header.updateTotalWidth(getOffsetWidth(), getTotalWidth());
    } else {
      updateColumnWidth(column, width);
      header.updateTotalWidth(getOffsetWidth(), getTotalWidth());
      if (GXT.isIE()) {
        syncHeaderScroll();
      }
    }

    // if (grid.isStateful()) {
    // Map<String, Object> state = grid.getState();
    // state.put("width" + cm.getColumnId(column), width);
    // grid.saveState();
    // }
  }

  protected void onDataChanged(StoreDataChangeEvent<M> se) {
    refresh(false);
    if (grid != null && grid.isLoadMask()) {
      if (grid.isEnabled()) {
        grid.unmask();
      } else {
        grid.mask();
      }
    }

    // if (grid != null && grid.isStateful() && ds != null && ds.getLoadConfig()
    // != null
    // && ds.getLoadConfig() instanceof PagingLoadConfig) {
    // PagingLoadConfig config = (PagingLoadConfig) ds.getLoadConfig();
    // Map<String, Object> state = grid.getState();
    // state.put("offset", config.getOffset());
    // state.put("limit", config.getLimit());
    // grid.saveState();
    // }
  }

  protected void onHeaderChange(int column, SafeHtml text) {
    header.setHeader(column, text);
  }

  protected void onHeaderClick(Grid<M> grid, int column) {
    this.headerColumnIndex = column;
    if (!headerDisabled && cm.isSortable(column)) {
      doSort(column, null);
    }
  }

  protected void onHiddenChange(ColumnModel<M> cm, int col, boolean hidden) {
    updateColumnHidden(col, hidden);

    // if (grid.isStateful()) {
    // Map<String, Object> state = grid.getState();
    // state.put("hidden" + cm.getColumnId(col), hidden);
    // grid.saveState();
    // }
  }

  protected void onHighlightRow(int rowIndex, boolean highlight) {
    Element row = getRow(rowIndex);
    if (row != null) {
      if (highlight) {
        addRowStyle(row, "x-grid3-highlightrow");
        if (Access.ariaIsEnabled()) {
          grid.setAriaState("aria-activedescendant", row.getId());
        }
      } else {
        removeRowStyle(row, "x-grid3-highlightrow");
      }
    }
  }

  protected void onMouseDown(Event ge) {

  }

  protected void onRemove(M m, int index, boolean isUpdate) {
    if (grid != null && grid.isViewReady()) {
      removeRow(index);
      if (!isUpdate) {
        removeTask.delay(10);
      } else {
        removeTask.delay(0);
      }
      constrainFocusElement();
    }
  }

  protected void onRowDeselect(int rowIndex) {
    Element row = getRow(rowIndex);
    if (row != null) {
      removeRowStyle(row, "x-grid3-row-selected");
      removeRowStyle(row, "x-grid3-highlightrow ");
      if (Access.ariaIsEnabled()) {
        row.setAttribute("aria-selected", "false");
      }
    }
  }

  protected void onRowOut(Element row) {
    if (grid.isTrackMouseOver()) {
      removeRowStyle(row, "x-grid3-row-over");
      overRow = null;
    }
  }

  protected void onRowOver(Element row) {
    if (grid.isTrackMouseOver()) {
      addRowStyle(row, "x-grid3-row-over");
      overRow = row;
    }
  }

  protected void onRowSelect(int rowIndex) {
    Element row = getRow(rowIndex);
    if (row != null) {
      onRowOut(row);
      addRowStyle(row, "x-grid3-row-selected");
      if (Access.ariaIsEnabled()) {
        row.setAttribute("aria-selected", "true");
        grid.getElement().setAttribute("aria-activedescendant", row.getId());
      }
    }
  }

  protected void onUpdate(ListStore<M> store, List<M> models) {
    for (M m : models) {
      refreshRow(store.indexOf(m));
    }
  }

  protected void processRows(int startRow, boolean skipStripe) {
    if (ds.size() < 1) {
      return;
    }
    skipStripe = skipStripe || !grid.isStripeRows();
    NodeList<Element> rows = getRows();
    String cls = "x-grid3-row-alt";
    for (int i = startRow, len = rows.getLength(); i < len; i++) {
      Element row = rows.getItem(i);
      row.setPropertyInt("rowIndex", i);
      if (!skipStripe) {
        boolean isAlt = (i + 1) % 2 == 0;
        boolean hasAlt = row.getClassName() != null && row.getClassName().indexOf(cls) != -1;
        if (isAlt == hasAlt) {
          continue;
        }
        if (isAlt) {
          fly(row).addClassName(cls);
        } else {
          fly(row).removeClassName(cls);
        }
      }
    }
  }

  protected void refreshRow(int row) {
    if (grid != null && grid.isViewReady()) {
      M m = ds.get(row);
      if (m != null) {
        // do not change focus on refresh
        // handles situation with changing cell value with field binding
        focusEnabled = false;

        insertRows(row, row, true);
        getRow(row).setPropertyInt("rowIndex", row);
        onRemove(m, row + 1, true);

        // GridEvent<ModelData> e = (GridEvent<ModelData>)
        // createComponentEvent(null);
        // e.setRowIndex(row);
        // e.setModel(ds.getAt(row));
        // fireEvent(Events.RowUpdated, e);
        focusEnabled = true;
      }
    }
  }

  protected void removeRow(int row) {
    Element r = getRow(row);
    if (r != null) {
      fly(r).removeFromParent();
    }
  }

  protected void removeRowStyle(Element row, String style) {
    fly(row).removeClassName(style);
  }

  protected void render() {
    renderUI();
    grid.sinkEvents(Event.ONCLICK | Event.ONDBLCLICK | Event.MOUSEEVENTS | Event.FOCUSEVENTS);
  }

  protected void renderFooter() {
    footer.setAllowTextSelection(false);

    mainWrap.appendChild(footer.getElement());
    footer.refresh();
  }

  protected void renderHeader() {
    XElement head = grid.getElement().selectNode(".x-grid3-hh");
    head.removeChildren();

    head.appendChild(header.getElement());
    header.refresh();

  }

  protected SafeHtml renderRows(int startRow, int endRow) {
    if (ds.size() < 1) {
      return SafeHtmlUtils.EMPTY_SAFE_HTML;
    }

    List<ColumnData> cs = getColumnData();

    if (endRow == -1) {
      endRow = ds.size() - 1;
    }

    List<M> rs = ds.subList(startRow, ++endRow);
    return doRender(cs, rs, startRow);
  }

  protected void renderUI() {
    String h = new StringBuilder(
        "<div class=\"x-grid3\" role=\"presentation\"><div class=\"x-grid3-viewport\" role=\"presentation\"><div class=\"x-grid3-header\" role=\"presentation\"><div class=\"x-grid3-header-inner\" role=\"presentation\"><div class=\"x-grid3-header-offset\" role=\"presentation\"><div class='x-grid3-hh' role='row'></div></div></div><div class=\"").append(
        CommonStyles.get().clear()).append(
        "\"></div></div><div class=\"x-grid3-scroller\" role=\"presentation\"><div class=\"x-grid3-body\" role=\"presentation\"></div><a href=\"#\" class=\"x-grid3-focus\" tabIndex=\"-1\"></a></div></div><div class=\"x-grid3-resize-marker\">&#160;</div><div class=\"x-grid3-resize-proxy\">&#160;</div></div>").toString();
    grid.getElement().setInnerHTML(h);

    renderHeader();

    initElements();

    header.setEnableColumnResizing(grid.isColumnResize());
    header.setEnableColumnReorder(grid.isColumnReordering());

    if (footer != null) {
      renderFooter();
    }

    updateHeaderSortState();
  }

  protected void resize() {
    if (mainBody == null) {
      return;
    }

    XElement c = grid.getElement();
    Size csize = c.getStyleSize();

    int vw = csize.getWidth();
    int vh = 0;
    if (vw < 10 || csize.getHeight() < 20) {
      return;
    }

    if (grid.isAutoHeight()) {
      el.setWidth(csize.getWidth());
      scroller.setWidth(vw);
    } else {
      el.setSize(csize.getWidth(), csize.getHeight());
    }

    int hdHeight = innerHd.getHeight(false);
    vh = csize.getHeight() - hdHeight;

    if (footer != null) {
      vh -= footer.getOffsetHeight();
    }

    if (!grid.isAutoHeight()) {
      scroller.setSize(vw, vh);
    }

    if (innerHd != null) {
      innerHd.setWidth(vw);
    }
    if (footer != null) {
      footer.setWidth(vw);
    }

  }

  protected void restoreScroll(Point state) {
    if (state.getY() < scroller.getWidth(false)) {
      scroller.setScrollLeft(state.getX());
    }
    if (state.getX() < scroller.getHeight(false)) {
      scroller.setScrollTop(state.getY());
    }
  }

  protected void syncHeaderScroll() {
    int sl = scroller.getScrollLeft();
    innerHd.setScrollLeft(sl);
    // second time for IE (1/2 time first fails, other browsers ignore)
    innerHd.setScrollLeft(sl);

    if (footer != null) {
      footer.getElement().setScrollLeft(sl);
      footer.getElement().setScrollLeft(sl);
    }
  }

  protected void syncHScroll() {
    if (!hasRows()) {
      XElement child = (XElement) mainBody.getFirstChildElement();

      if (child != null) {
        child.setWidth(cm.getTotalWidth(), true);
      }
    }
  }

  protected void syncScroll() {
    syncHeaderScroll();
    int scrollLeft = scroller.getScrollLeft();
    int scrollTop = scroller.getScrollTop();
    constrainFocusElement();
    grid.fireEvent(new BodyScrollEvent(scrollLeft, scrollTop));
  }

  protected void templateOnAllColumnWidthsUpdated(Stack<Integer> ws, int tw) {

  }

  protected void templateOnColumnHiddenUpdated(int col, boolean hidden, int tw) {
    // template method
  }

  protected void templateOnColumnWidthUpdated(int col, int w, int tw) {
    // template method
  }

  protected void updateAllColumnWidths() {
    int tw = getTotalWidth();
    int clen = cm.getColumnCount();
    Stack<Integer> ws = new Stack<Integer>();

    for (int i = 0; i < clen; i++) {
      ws.push(getColumnWidth(i));
      header.updateColumnWidth(i, cm.getColumnWidth(i));
      if (footer != null) {
        footer.updateColumnWidth(i, cm.getColumnWidth(i));
      }
    }

    NodeList<Element> ns = getRows();
    for (int i = 0, len = ns.getLength(); i < len; i++) {
      Element row = ns.getItem(i);
      row.getStyle().setPropertyPx("width", tw);
      if (row.getFirstChild() != null) {
        row.getFirstChildElement().getStyle().setPropertyPx("width", tw);
        TableSectionElement e = row.getFirstChild().cast();
        TableRowElement nodeList = e.getRows().getItem(0);
        for (int j = 0; j < clen; j++) {
          ((Element) nodeList.getChildNodes().getItem(j)).getStyle().setPropertyPx("width", ws.get(j));
        }
      }
    }

    templateOnAllColumnWidthsUpdated(ws, tw);
    syncHScroll();
  }

  protected void updateColumnHidden(int index, boolean hidden) {
    int tw = getTotalWidth();
    String display = hidden ? "none" : "";

    fly(innerHd.getFirstChildElement()).setWidth(getOffsetWidth());
    fly(innerHd.getFirstChildElement().getFirstChildElement()).setWidth(tw);

    header.updateColumnHidden(index, hidden);
    if (footer != null) {
      footer.updateTotalWidth(getOffsetWidth(), tw);
      footer.updateColumnHidden(index, hidden);
    }

    NodeList<Element> ns = getRows();
    for (int i = 0, len = ns.getLength(); i < len; i++) {
      Element elem = ns.getItem(i);
      elem.getStyle().setProperty("width", tw + "px");
      TableSectionElement e = (TableSectionElement) elem.getFirstChild();
      if (e != null) {
        e.getStyle().setProperty("width", tw + "px");
        Element cell = e.getRows().getItem(0).getChildNodes().getItem(index).cast();
        cell.getStyle().setProperty("display", display);
      }
    }

    templateOnColumnHiddenUpdated(index, hidden, tw);

    lastViewWidth = -1;
    layout();
    syncHScroll();
  }

  protected void updateColumnWidth(int col, int width) {
    int tw = getTotalWidth();
    int w = getColumnWidth(col);

    header.updateTotalWidth(-1, tw);
    header.updateColumnWidth(col, width);

    if (footer != null) {
      footer.updateTotalWidth(getOffsetWidth(), tw);
      footer.updateColumnWidth(col, width);
    }

    NodeList<Element> ns = getRows();
    for (int i = 0, len = ns.getLength(); i < len; i++) {
      Element row = ns.getItem(i);
      row.getStyle().setPropertyPx("width", tw);
      if (row.getFirstChild() != null) {
        row.getFirstChildElement().getStyle().setPropertyPx("width", tw);
        TableSectionElement e = row.getFirstChild().cast();
        ((Element) e.getRows().getItem(0).getChildNodes().getItem(col)).getStyle().setPropertyPx("width", w);
      }
    }

    templateOnColumnWidthUpdated(col, w, tw);
    syncHScroll();
  }

  protected void updateHeaderSortState() {
    if (!isRemoteSort()) {
      StoreSortInfo<M> info = getSortState();
      if (info != null) {
        ValueProvider<M, ?> vp = info.getValueProvider();
        if (vp != null) {
          String p = vp.getPath();
          if (p != null && !"".equals(p)) {
            ColumnConfig<M, ?> config = cm.findColumnConfig(p);
            if (config != null) {
              if (storeSortInfo == null || (!p.equals(storeSortInfo.getPath()))
                  || storeSortInfo.getDirection() != info.getDirection()) {
                int index = cm.indexOf(config);
                if (index != -1) {
                  updateSortIcon(index, info.getDirection());
                }
                grid.fireEvent(new SortChangeEvent(new SortInfoBean(info.getPath(), info.getDirection())));
              }

            }
          }
          storeSortInfo = info;
        }

      }
    } else {
      List<? extends SortInfo> infos = grid.getLoader().getSortInfo();
      if (infos.size() > 0) {
        SortInfo info = infos.get(0);
        String p = info.getSortField();
        if (p != null && !"".equals(p)) {
          ColumnConfig<M, ?> config = cm.findColumnConfig(p);
          if (config != null) {
            if (sortState == null || (!sortState.getSortField().equals(p))
                || sortState.getSortDir() != info.getSortDir()) {
              int index = cm.indexOf(config);
              if (index != -1) {
                updateSortIcon(index, info.getSortDir());
              }
              grid.fireEvent(new SortChangeEvent(info));
            }
          }
          sortState = info;
        }
      }
    }
  }

  protected void updateSortIcon(int colIndex, SortDir dir) {
    header.updateSortIcon(colIndex, dir);
  }

  private void constrainFocusElement() {
    int scrollLeft = scroller.getScrollLeft();
    int scrollTop = scroller.getScrollTop();
    int left = scroller.getWidth(true) / 2 + scrollLeft;
    int top = scroller.getHeight(true) / 2 + scrollTop;
    focusEl.setLeftTop(left, top);
  }

  private void refreshFooterData() {
    if (footer != null) {
      footer.refresh();
    }
  }

  private void restrictMenu(Menu columns) {
    int count = 0;
    for (int i = 0, len = cm.getColumnCount(); i < len; i++) {
      if (!shouldNotCount(i, true)) {
        count++;
      }
    }

    if (count == 1) {
      for (Widget item : columns) {
        CheckMenuItem ci = (CheckMenuItem) item;
        if (ci.isChecked()) {
          ci.disable();
        }
      }
    } else {
      for (Widget item : columns) {
        if (item instanceof Component) {
          ((Component) item).enable();
        }
      }
    }
  }

  private boolean shouldNotCount(int columnIndex, boolean includeHidden) {
    return cm.getColumnHeader(columnIndex) == null || cm.getColumnHeader(columnIndex).asString().equals("")
        || (includeHidden && cm.isHidden(columnIndex)) || cm.isFixed(columnIndex);
  }

}
