package com.sencha.gxt.widget.core.client.grid.editing;

import com.google.gwt.core.client.Scheduler;
import com.google.gwt.core.client.Scheduler.ScheduledCommand;
import com.google.gwt.dom.client.Element;
import com.google.gwt.dom.client.NativeEvent;
import com.google.gwt.event.dom.client.ScrollEvent;
import com.google.gwt.event.logical.shared.ValueChangeEvent;
import com.google.gwt.event.logical.shared.ValueChangeHandler;
import com.google.gwt.event.shared.HandlerRegistration;
import com.sencha.gxt.core.client.ValueProvider;
import com.sencha.gxt.core.client.util.KeyNav;
import com.sencha.gxt.core.shared.event.CancellableEvent;
import com.sencha.gxt.data.shared.Converter;
import com.sencha.gxt.data.shared.ListStore;
import com.sencha.gxt.widget.core.client.ComponentHelper;
import com.sencha.gxt.widget.core.client.event.BlurEvent;
import com.sencha.gxt.widget.core.client.event.BlurEvent.BlurHandler;
import com.sencha.gxt.widget.core.client.form.Field;
import com.sencha.gxt.widget.core.client.grid.ColumnConfig;
import com.sencha.gxt.widget.core.client.grid.Grid;
import com.sencha.gxt.widget.core.client.grid.Grid.GridCell;

public class GridInlineEditing<M> extends AbstractGridEditing<M> {

  protected class GridEditingKeyNav extends AbstractGridEditingKeyNav {

    @Override
    public void onTab(NativeEvent evt) {
      GridInlineEditing.this.onTab(evt);
    }
  }

  protected HandlerRegistration fieldBlurHandlerRegistration;
  protected boolean ignoreScroll;

  private boolean ignoreBlur;

  public GridInlineEditing(Grid<M> editableGrid) {
    setEditableGrid(editableGrid);
  }

  @Override
  public void cancelEditing() {
    ignoreScroll = false;
    if (activeCell != null) {

      ColumnConfig<M, ?> c = getEditableGrid().getColumnModel().getColumn(activeCell.getCol());
      Field<?> field = getEditor(c);
      removeEditor(activeCell, field);

      final GridCell gc = activeCell;
      activeCell = null;

      fireEvent(new CancelEditEvent<M>(gc));
    }
  }

  @Override
  public void completeEditing() {
    if (activeCell != null) {
      ColumnConfig<M, ?> c = getEditableGrid().getColumnModel().getColumn(activeCell.getCol());

      Field<?> field = getEditor(c);

      if (field != null) {
        // commit editor value
        getEditableGrid().getView().focusCell(activeCell.getRow(), activeCell.getCol(), false);
      }
      Scheduler.get().scheduleDeferred(new ScheduledCommand() {
        @Override
        public void execute() {
          doCompleteEditing();
        }
      });
    }
  }

  public void startEditing(final GridCell cell) {
    ignoreBlur = false;
    if (getEditableGrid() != null && getEditableGrid().isAttached() && cell != null) {
      ColumnConfig<M, ?> c = getEditableGrid().getColumnModel().getColumn(cell.getCol());

      M value = getEditableGrid().getStore().get(cell.getRow());
      // editable
      if (value != null && getEditor(c) != null) {
        CancellableEvent ce = new BeforeStartEditEvent<M>(cell);
        if (ce.isCancelled()) {
          return;
        }

        cancelEditing();
        ignoreScroll = true;
        getEditableGrid().getView().ensureVisible(cell.getRow(), cell.getCol(), true);
        Scheduler.get().scheduleDeferred(new ScheduledCommand() {

          @Override
          public void execute() {
            doStartEditing(cell);
          }
        });

      }
    }

  }

  @SuppressWarnings("unchecked")
  protected <N, O> void doCompleteEditing() {
    if (activeCell != null) {
      ColumnConfig<M, N> c = getEditableGrid().getColumnModel().getColumn(activeCell.getCol());

      Field<O> field = getEditor(c);

      if (field != null) {

        Converter<N, O> converter = getConverter(c);

        O fieldValue = field.getValue();

        N convertedValue;
        if (converter != null) {
          convertedValue = converter.convertFieldValue(fieldValue);
        } else {
          convertedValue = (N) fieldValue;
        }

        ListStore<M> store = getEditableGrid().getStore();
        ListStore<M>.Record r = store.getRecord(store.get(activeCell.getRow()));
        r.addChange(c.getValueProvider(), convertedValue);

        removeEditor(activeCell, field);
      }
      final GridCell gc = activeCell;
      activeCell = null;
      fireEvent(new CompleteEditEvent<M>(gc));
    }
  }

  @SuppressWarnings("unchecked")
  protected <N, O> void doStartEditing(final GridCell cell) {
    if (getEditableGrid() != null && getEditableGrid().isAttached() && cell != null) {
      M value = getEditableGrid().getStore().get(cell.getRow());

      ColumnConfig<M, N> c = getEditableGrid().getColumnModel().getColumn(cell.getCol());
      if (c != null && value != null) {

        Converter<N, O> converter = getConverter(c);

        ValueProvider<M, N> v = c.getValueProvider();
        N colValue = getEditableGrid().getStore().hasRecord(value)
            ? getEditableGrid().getStore().getRecord(value).getValue(v) : v.getValue(value);
        O convertedValue;
        if (converter != null) {
          convertedValue = converter.convertModelValue(colValue);
        } else {
          convertedValue = (O) colValue;
        }

        final Field<O> field = getEditor(c);
        if (field != null) {
          activeCell = cell;
          field.setValue(convertedValue);

          getEditableGrid().getView().getEditorParent().appendChild(field.getElement());
          ComponentHelper.setParent(getEditableGrid(), field);
          ComponentHelper.doAttach(field);

          field.setWidth(c.getWidth());
          field.getElement().makePositionable(true);

          Element row = getEditableGrid().getView().getRow(cell.getRow());

          int left = 0;
          for (int i = 0; i < cell.getCol(); i++) {
            left += getEditableGrid().getColumnModel().getColumnWidth(i);
          }

          field.getElement().setLeftTop(left, row.getOffsetTop());

          Scheduler.get().scheduleDeferred(new ScheduledCommand() {
            @Override
            public void execute() {
              field.focus();
              GridInlineEditing.this.ignoreScroll = false;

              field.addValueChangeHandler(new ValueChangeHandler<O>() {

                @Override
                public void onValueChange(ValueChangeEvent<O> event) {

                }
              });

              GridInlineEditing.this.fieldBlurHandlerRegistration = field.addBlurHandler(new BlurHandler() {

                @Override
                public void onBlur(final BlurEvent event) {
                  if (GridInlineEditing.this.fieldBlurHandlerRegistration != null) {

                    GridInlineEditing.this.onBlurField(event);
                  }

                }
              });
              fireEvent(new StartEditEvent<M>(cell));
            }
          });
        }
      }
    }

  }

  @Override
  protected KeyNav ensureInternalKeyNav() {
    if (keyNav == null) {
      keyNav = new GridEditingKeyNav();
    }
    return keyNav;
  }

  protected void onBlurField(BlurEvent event) {
    Scheduler.get().scheduleFinally(new ScheduledCommand() {
      @Override
      public void execute() {
        if (!ignoreBlur && isEditing()) {

          ignoreBlur = false;
          final GridCell cell = activeCell;
          cancelEditing();
          Scheduler.get().scheduleDeferred(new ScheduledCommand() {

            @Override
            public void execute() {
              getEditableGrid().getView().focusCell(cell.getRow(), cell.getCol(), true);

            }
          });

        }
      }
    });
  }

  @Override
  protected void onEnter(NativeEvent evt) {
    ignoreBlur = true;
    super.onEnter(evt);
  }

  protected void onScroll(ScrollEvent event) {
    if (!ignoreScroll) {
      cancelEditing();
    }
  }

  protected void onTab(NativeEvent evt) {
    ignoreBlur = true;
    evt.preventDefault();
    if (activeCell != null) {
      GridCell cell = activeCell;

      GridCell newCell = null;

      completeEditing();

      if (evt.getShiftKey()) {
        newCell = getEditableGrid().walkCells(cell.getRow(), cell.getCol() - 1, -1, callback);
      } else {
        newCell = getEditableGrid().walkCells(cell.getRow(), cell.getCol() + 1, 1, callback);
      }
      if (newCell != null) {
        final GridCell c = newCell;
        Scheduler.get().scheduleDeferred(new ScheduledCommand() {

          @Override
          public void execute() {
            startEditing(c);

          }
        });
      } else {
        ignoreBlur = false;
        getEditableGrid().getView().focusCell(cell.getRow(), cell.getCol(), true);
      }
    }

  }

  private void removeEditor(final GridCell cell, final Field<?> field) {
    removeFieldBlurHandler();
    if (field != null && field.isAttached()) {
      field.redraw();
      ComponentHelper.setParent(null, field);
      ComponentHelper.doDetach(field);
      field.getElement().removeFromParent();
    }
  }

  private void removeFieldBlurHandler() {
    if (fieldBlurHandlerRegistration != null) {
      fieldBlurHandlerRegistration.removeHandler();
      fieldBlurHandlerRegistration = null;
    }
  }

}
